-- Copyright 2022 by Todd Hundersmarck (ThundR)
-- All Rights Reserved
THAnimalMoveEvent = { ERROR_CODE = { SUCCESS = 0, NO_PERMISSION = 1, INVALID_SOURCE = 2, INVALID_TARGET = 3, INVALID_CLUSTER = 4, NOT_ENOUGH_SPACE = 5, ANIMAL_NOT_SUPPORTED = 6, INVALID_NUM_ANIMALS = 7, INVALID_FARM = 8, INTERNAL = 9 }, ERROR_NUM_BITS = 4 } local THAnimalMoveEvent_mt = Class(THAnimalMoveEvent, Event) InitEventClass(THAnimalMoveEvent, "THAnimalMoveEvent") local function runScript() end function THAnimalMoveEvent.emptyNew() local newEnv = Event.new(THAnimalMoveEvent_mt) return newEnv end function THAnimalMoveEvent.new(sourceObject, targetObject, clusterId, numAnimals, isReversed) local newEnv = THAnimalMoveEvent.emptyNew() if newEnv then newEnv.sourceObject = sourceObject newEnv.targetObject = targetObject newEnv.clusterId = clusterId newEnv.numAnimals = numAnimals newEnv.isReversed = Utils.getNoNil(isReversed, false) end return newEnv end function THAnimalMoveEvent.newServerToClient(errorCode) local newEnv = THAnimalMoveEvent.emptyNew() if newEnv then newEnv.errorCode = errorCode end return newEnv end function THAnimalMoveEvent:readStream(streamId, connection) if not connection:getIsServer() then self.sourceObject = NetworkUtil.readNodeObject(streamId) self.targetObject = NetworkUtil.readNodeObject(streamId) self.clusterId = streamReadInt32(streamId) self.numAnimals = streamReadUIntN(streamId, 10) self.isReversed = streamReadBool(streamId) else self.errorCode = streamReadUIntN(streamId, THAnimalMoveEvent.ERROR_NUM_BITS) end self:run(connection) end function THAnimalMoveEvent:writeStream(streamId, connection) if connection:getIsServer() then NetworkUtil.writeNodeObject(streamId, self.sourceObject) NetworkUtil.writeNodeObject(streamId, self.targetObject) streamWriteInt32(streamId, self.clusterId) streamWriteUIntN(streamId, self.numAnimals, 10) streamWriteBool(streamId, self.isReversed) else streamWriteUIntN(streamId, self.errorCode, THAnimalMoveEvent.ERROR_NUM_BITS) end end function THAnimalMoveEvent:run(connection) if not connection:getIsServer() then local uniqueUserId = g_currentMission.userManager:getUniqueUserIdByConnection(connection) local farm = g_farmManager:getFarmForUniqueUserId(uniqueUserId) local farmId = farm.farmId local errorCode = THAnimalMoveEvent.validate(self.sourceObject, self.targetObject, self.clusterId, self.numAnimals, self.isReversed, farmId) if errorCode then connection:sendEvent(THAnimalMoveEvent.newServerToClient(errorCode)) return end local trailer = self.sourceObject local trigger = self.targetObject if self.isReversed then trailer = self.targetObject trigger = self.sourceObject end local numAnimals = self.numAnimals if self.isReversed then numAnimals = -numAnimals end local cluster = trailer:getClusterById(self.clusterId) local clusterSystem = trailer:getClusterSystem() local subTypeInfo = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex) local fillTypeIndex = subTypeInfo.fillTypeIndex cluster:changeNumAnimals(-numAnimals) clusterSystem:updateNow() trigger:addFillUnitFillLevel(farmId, nil, numAnimals, fillTypeIndex, ToolType.UNDEFINED) connection:sendEvent(THAnimalMoveEvent.newServerToClient(THAnimalMoveEvent.ERROR_CODE.SUCCESS)) else g_messageCenter:publish(THAnimalMoveEvent, self.errorCode) end end function THAnimalMoveEvent.validate(sourceObject, targetObject, clusterId, numAnimals, isReversed, farmId) local function protectedFunc() if type(numAnimals) ~= "number" or numAnimals < 0 then return THAnimalMoveEvent.ERROR_CODE.INVALID_NUM_ANIMALS end if not g_farmManager:getFarmById(farmId) then return THAnimalMoveEvent.ERROR_CODE.INVALID_FARM end if isReversed and isReversed ~= true then return THAnimalMoveEvent.ERROR_CODE.INTERNAL end local trailer = sourceObject local trigger = targetObject if isReversed then trailer = targetObject trigger = sourceObject end local trailerIsValid = trailer ~= nil and trailer.getClusterById ~= nil and trailer.getClusterSystem ~= nil local triggerIsValid = false local triggerData = g_thMain:getDataTable(trigger) if triggerData then local animalTriggerData = g_thMain:getDataTable(triggerData.animalTrigger) if animalTriggerData and animalTriggerData.isProductionTrigger then triggerIsValid = true end end if not trailerIsValid or (isReversed and not triggerIsValid) then return THAnimalMoveEvent.ERROR_CODE.INVALID_SOURCE elseif not triggerIsValid or (isReversed and not trailerIsValid) then return THAnimalMoveEvent.ERROR_CODE.INVALID_TARGET end local cluster = trailer:getClusterById(clusterId) if not cluster then return THAnimalMoveEvent.ERROR_CODE.INVALID_CLUSTER end local station = trigger:getTarget() local subTypeIndex = cluster.subTypeIndex local subTypeInfo = g_currentMission.animalSystem:getSubTypeByIndex(subTypeIndex) local fillTypeIndex = subTypeInfo.fillTypeIndex local freeCapacity = 0 if isReversed then if not trailer:getSupporsAnimalSubType(subTypeIndex) then return THAnimalMoveEvent.ERROR_CODE.ANIMAL_NOT_SUPPORTED elseif not g_currentMission.accessHandler:canFarmAccess(farmId, trailer) then return THAnimalMoveEvent.ERROR_CODE.NO_PERMISSION else freeCapacity = trailer:getNumOfFreeAnimalSlots(subTypeIndex) end else if not trigger:getIsFillTypeSupported(fillTypeIndex) then return THAnimalMoveEvent.ERROR_CODE.ANIMAL_NOT_SUPPORTED elseif not g_currentMission.accessHandler:canFarmAccess(farmId, station) then return THAnimalMoveEvent.ERROR_CODE.NO_PERMISSION else freeCapacity = trigger:getFillUnitFreeCapacity(nil, fillTypeIndex, farmId) end end if numAnimals > freeCapacity then return THAnimalMoveEvent.ERROR_CODE.NOT_ENOUGH_SPACE end end local success, errorCode = g_thMain:pcall(protectedFunc) if not success then errorCode = THAnimalMoveEvent.ERROR_CODE.INTERNAL end return errorCode end runScript()